# -*- coding: utf-8 -*-
require 'nokogiri'
require 'uri'
require 'json'
require 'baiduserp/client'
require 'baiduserp/helper'
require 'baiduserp/result'

module Baiduserp
  class Parser
    Dir[File.expand_path('../parser/*.rb', __FILE__)].each{|f| require f}

    def parse(html)
      html = html.encode!('UTF-8','UTF-8',:invalid => :replace)
      @file = Hash.new
      @serp = Baiduserp::Result.new

      @file[:html] = html
      @file[:doc] = Nokogiri::HTML(html)

      self.methods.each do |m|
        next unless m =~ /^_parse_/
        #p m
        begin
          @serp[m.to_s.sub('_parse_','').to_sym] = self.send m,@file
        rescue Exception => e
          issue_file = "/tmp/baiduserp_issue_#{Time.now.strftime("%Y%m%d%H%M%S")}.html"
          open(issue_file,'w').puts(html)
          puts "Notice:"
          puts "Baiduserp gem have a bug, please email to zmingqian@qq.com to report it."
          puts "Please attach file #{issue_file} in the email and the error information below, thanks!"
          puts e.message
          puts e.inspect
          puts e.backtrace
          raise "Baiduserp Parser Get An Error!"
        end
        #p @serp.keys
      end

      @serp
    end

    def search(keyword,page=1)
      html = get_search_html(keyword,page)
      parse html
    end

    def get_search_html(keyword,page=1)
      keyword = keyword.gsub(" ","+")
      page = page.to_i > 1 ? "&pn=#{page.to_i-1}0" : ""
      serp_url = URI.escape("http://www.baidu.com/s?wd=#{keyword}#{page}&ie=utf-8&inputT=#{1000+rand(1000)}")
      # serp_url = URI.escape("http://www.baidu.com/s?wd=#{keyword}#{page}&ie=utf-8")
      Client.get_serp(serp_url).body
    end

    def parse_file(file_path)
      if File.exists? file_path
        html = open(file_path).read
      else
        html = Client.get_serp(file_path).body
      end
      parse html
    end
  end
end
